/*	controller.js
	Another class name we could use here would be Router
*/

const
	UrlWrapper = require('url-wrapper').UrlWrapper,
	TemperatureConverter = require('./model/temperature-converter').TemperatureConverter,			
	LengthConverter = require('./model/length-converter').LengthConverter,	
	EnglishToSpanishTranslator = require('./model/english-translator').EnglishToSpanishTranslator,	
	statusCodes = require('http-status-codes'),
	log = console.log,
	htmlContentType = {'Content-Type': 'text/html'},
	jsonType = {'Content-Type': 'application/json'},	
	plainType = {"Content-Type": "text/plain"},	
	br = '<br>';

//=====================================	

class Controller {
	
	constructor(request, response) {
		this.url = UrlWrapper.fromRequest(request);
		this.request = request;
		this.response = response;
	}
	
	//-------------------------------------
	//public
	
	processRequest() {
		if (!this.authenticate())
			return;	
		this.logRequestInfo();
		this.route();
	}
	
	route() {
		const path = this.url.path();
		if (path === '/convert/temperature')
			this.convert(TemperatureConverter);
		else if (path === '/convert/length')
			this.convert(LengthConverter);		
		else if (path === '/translate/english')
			this.translateEnglish();	
		else
			log(`URL Path not handled: ${path}`);			
	}
	
	//-------------------------------------
	//private
	
	convert(converterClass) {
		const 
			query = this.url.query(),
			unit = query.unit,
			value = query.value;		
		if (!this.validateParam('unit', unit) || !this.validateParam('value', value))
			return;						
		const converter = new converterClass(unit, value);
		this.serve(converter.getResults());
	}
	
	translateEnglish() {
		const englishWord = this.url.query().word;	
		if (!this.validateParam('word', englishWord))
			return;
		const translator = new EnglishToSpanishTranslator(englishWord);
		this.serve(translator.getResults());
	}	

	//-------------------------------------
	//validation - private
	
	validateParam(paramName, value) {
		if (value === undefined) {
			const res = this.response;
			res.writeHead(statusCodes.BAD_REQUEST, plainType);
			res.end('ERROR: ' + `Missing query param "${paramName}"`);
			return false;
		}
		return true; //ok
	}
	
	authenticate() {
		//check API key
		//simplified authentication to exemplify
		const
			expected = 'Z283794',
			res = this.response;
		if (this.url.query().apiKey !== expected) {
			res.writeHead(statusCodes.BAD_REQUEST, plainType);
			res.end("Invalid API key\n");
			return false;
		}
		return true;		
	}	
	
	//-------------------------------------
	//serving data - private	
	
	serve(results) {
		//serve in JSON format
		const res = this.response;
		res.writeHead(statusCodes.OK, jsonType);		
		res.end(JSON.stringify(results));
	}		
	
	//-------------------------------------
	//helpers - private		
	
	logRequestInfo() {
		//dump request info to console
		log('received url: ' + this.request.url);	
		log('url path: ' + this.url.path());
		//stringify converts to user-friendly format
		log('url query: ' + JSON.stringify(this.url.query()));			
	}	
	
}
	
//=====================================	

exports.Controller = Controller;	
	
	
	
